#include <fcntl.h>
#include <io.h>
#include "HiewixExport.h"

// compile line:
// cl DemoDosHeader.cpp /link /OUT:"DemoDosHeader.hwx32" "kernel32.lib" "HxDemo.lib" /DLL /SUBSYSTEM:WINDOWS


// 3 necessarily functions:
// ApiVersionUsed()
// IKnowThisFile()
// ParseFile()

/////////////////////////////////////////////////////////////////////////////

 long __stdcall  DllMain( void *hModule, unsigned long reason, void *lpReserved ){ return 1; }

/////////////////////////////////////////////////////////////////////////////

typedef struct{
    unsigned short   e_magic;    // Magic number
    unsigned short   e_cblp;     // Bytes on last page of file
    unsigned short   e_cp;       // Pages in file
    unsigned short   e_crlc;     // Relocations
    unsigned short   e_cparhdr;  // Size of header in paragraphs
    unsigned short   e_minalloc; // Minimum extra paragraphs needed
    unsigned short   e_maxalloc; // Maximum extra paragraphs needed
    unsigned short   e_ss;       // Initial SS value
    unsigned short   e_sp;       // Initial SP value
    unsigned short   e_csum;     // Checksum
    unsigned short   e_ip;       // Initial IP value
    unsigned short   e_cs;       // Initial CS value
    unsigned short   e_lfarlc;   // File address of relocation table
}DOS_HEADER;

/////////////////////////////////////////////////////////////////////////////

 bool  ReadHeader( const wchar_t *pfilename, long *pfilesize, DOS_HEADER *p_dos_header )
{
 int  h =  ::_wopen( pfilename, _O_RDONLY | _O_BINARY );
 if(  h == -1 )
   return false;

 if( ::_read( h, p_dos_header, sizeof(DOS_HEADER) ) != sizeof(DOS_HEADER) )
   p_dos_header->e_magic = 0;

 *pfilesize = ::_filelength( h );

 ::_close( h );

 return p_dos_header->e_magic == 0x5A4D 
     || p_dos_header->e_magic == 0x4D5A;
}

/////////////////////////////////////////////////////////////////////////////
// 1st necessarily function
 HWX_EXPORT int  ApiVersionUsed()
{
 return HWX_API_VERSION;
}

/////////////////////////////////////////////////////////////////////////////
// 2nd necessarily function
 HWX_EXPORT bool  IKnowThisFile( HWX_PREFILE_DATA *p_hwx_prefile_data )
{
 long        filesize;
 DOS_HEADER  dos_header;
 if( ReadHeader( p_hwx_prefile_data->_filename, &filesize, &dos_header ) == false )
   return false;                                                               // not my file

 p_hwx_prefile_data->_file_flags    = file_dialog_data_little_endian           // only LE
                                    | file_dialog_data_enable                  // file may be as data
                                    | file_dialog_code16_enable;               // file may be as code 16bits 
 p_hwx_prefile_data->_base_address  = 0;                                       // base address
 p_hwx_prefile_data->_codesize      = 0;                                       // first: file as data
 p_hwx_prefile_data->_decode_engine = decode_engine_ia32;                      // disassembler for code, decode_engine_none if no code 

 return true;                                                                  // yes, i'm know
}

/////////////////////////////////////////////////////////////////////////////
// 3rd necessarily function
 HWX_EXPORT bool  ParseFile( HWX_FILE_DATA *p_hwx_file_data )
{
 long        filesize;
 DOS_HEADER  dos_header;
 ReadHeader( p_hwx_file_data->_filename, &filesize, &dos_header );

 const int file_index = p_hwx_file_data->_file_index; 

 HwxFileLog( file_index, L"DemoDosHeader is running..." );                     // line to log pane

//
//  Only one segment in DEMO
//
 long  segment_size = dos_header.e_cp * 512/*filepage*/ + dos_header.e_cblp;
 if(   segment_size > filesize )
   segment_size = filesize;
 if( p_hwx_file_data->_codesize == 0 )                                         // user select file as data..
   HwxFileAddDataSegment(  file_index, 0, 0, segment_size, 0 );                // ..define data segment
  else                                                                         // or
   HwxFileAddCodeSegment(  file_index, 0, 0, segment_size, 0 );                // ..define code segment
 HwxFileSegmentName(       file_index, 0, L"DEMO" );                           // lets name for segment

//
//  Header
//
 HwxFileSetRunAddress(    file_index, 0 );                                     // runaddress start at 0
 HwxFileRunCommentline(   file_index, L"Header" );                             // commentline for runaddress
 HwxFileRunCommentOnline( file_index, L"Magic number" );                       // comment online for runaddress
 HwxFileRunWord(          file_index );                                        // one WORD; runaddress += sizeof WORD
 HwxFileMakeDec(          file_index );                                        // show next numbers as decimal
 HwxFileRunCommentOnline( file_index, L"Bytes on last page of file"       );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Pages in file"                    );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Relocations"                      );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Size of header in paragraphs"     );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Minimum extra paragraphs needed"  );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Maximum extra paragraphs needed"  );  
 HwxFileRunWord(          file_index );
 HwxFileMakeHex(          file_index );                                        // show next numbers as hex
 HwxFileRunCommentOnline( file_index, L"Initial SS value"                 );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Initial SP value"                 );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Checksum"                         );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Initial IP value"                 );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"Initial CS value"                 );  
 HwxFileRunWord(          file_index );
 HwxFileRunCommentOnline( file_index, L"File address of relocation table" );  
 HwxFileRunWord(          file_index );

//
// Relocations
//
 if( dos_header.e_crlc ){
   const int offset_segment = 1+1/*word + word*/;
   HwxFileCommentline(  file_index, dos_header.e_lfarlc, L"Relocations (offset:segment)" );                          // comment line at address
   HwxFileMakePerline(  file_index, offset_segment );                                                                // numbers per line
   HwxFileMakeWords(    file_index, dos_header.e_lfarlc, dos_header.e_crlc * offset_segment );                       // list of WORDs
   HwxFileMakeCollapse( file_index, dos_header.e_lfarlc, dos_header.e_crlc * offset_segment * sizeof(short), true ); // define region of bytes and collapse it
 }

//
// Body
//
 long  body_offset = dos_header.e_cparhdr * 16/*paragraph*/;
 long  body_size   = segment_size - body_offset;
 HwxFileCommentline( file_index, body_offset, L"Body" );                       // comment line at address
 if( p_hwx_file_data->_codesize == 0 )                                         // user select file as data..
   HwxFileMakeDump( file_index, body_offset, body_size, false );               // ..show as dump
  else                                                                         // or
   HwxFileMakeCode( file_index, body_offset );                                 // ..show as code

 return true;                                                                  // parsing is success
}

////////////////////////////////////////////////////////////
