#pragma once

const int HWX_API_VERSION = 2503;

#pragma pack(push)
#pragma pack(1)
typedef struct _HWX_PREFILE_DATA{
  const int                            _filename_size_in_bytes;
  const int                            _r1;
        int                            _file_flags;
        short                          _codesize;
        short                          _decode_engine;
        __int64                        _base_address;
  const void                          *_r2[4];
  const wchar_t                        _filename[1];
  }HWX_PREFILE_DATA;

typedef struct _HWX_FILE_DATA{
  const int                            _filename_size_in_bytes;
  const int                            _file_index;
  const int                            _file_flags;
  const short                          _codesize;
  const short                          _decode_engine;
  const __int64                        _base_address;
  const void                          *_r2[4];
        void                          *_p_second_parser_data;
        void                          *_p_main_parser_data;
  const wchar_t                        _filename[1];
  }HWX_FILE_DATA;
#pragma pack()

enum{ // HwxFlags()
  hwx_second_parser                   = 0x0002,
};

enum{ // _file_flags
  file_parser_is_reenterable          = 0x00000010,
  file_dialog_data_enable             = 0x00000100,
  file_dialog_code16_enable           = 0x00000200,
  file_dialog_code32_enable           = 0x00000400,
  file_dialog_code64_enable           = 0x00000800,
  file_dialog_address_enable          = 0x00001000,
  file_dialog_data_little_endian      = 0x00002000,
  file_dialog_data_big_endian         = 0x00004000,
  file_dialog_scan_refers             = 0x00008000,
  file_dialog_scan_wide_odd           = 0x00100000,
  file_dialog_scan_wide_even          = 0x00200000,
  file_dialog_scan_ascii              = 0x00400000,
  file_dialog_scan_latin_only         = 0x00800000,
  file_dialog_scan_ascii_latin_only   = (file_dialog_scan_ascii|file_dialog_scan_latin_only),
};

enum decode_engine_t{
  decode_engine_none                  = 0,
  decode_engine_ia32                  = 1,
};

enum bookmark_color_t{
  bookmark_red                        = 0,
  bookmark_blue,
  bookmark_green,
  bookmark_yellow,
};

enum string_ascii_cp_t{
  ascii_cp_ansi                       = 0,
  ascii_cp_oem,
  ascii_cp_utf8,
};

#define HWX_EXPORT extern "C" __declspec(dllexport)
#ifndef HWX_API
#define HWX_API    extern "C" __declspec(dllimport)
#endif

//  necessarily
HWX_EXPORT int                  ApiVersionUsed();
HWX_EXPORT bool                 IKnowThisFile(            HWX_PREFILE_DATA *p_hwx_prefile_data );
HWX_EXPORT bool                 ParseFile(                HWX_FILE_DATA    *p_hwx_file_data );

HWX_EXPORT unsigned long        HwxID();  // above 0xFFFF
HWX_EXPORT const wchar_t       *HwxTitle();
//HWX_EXPORT unsigned long        HwxFlags();
//HWX_EXPORT void                 OnFileOpen(                HWX_FILE_DATA    *p_hwx_file_data );
//HWX_EXPORT void                 OnFileClose(               HWX_FILE_DATA    *p_hwx_file_data );
//HWX_EXPORT __int64              ToAddress(           const HWX_FILE_DATA    *p_hwx_file_data, __int64 qw );
//HWX_EXPORT int                  VirtualRead(         const HWX_FILE_DATA    *p_hwx_file_data, __int64 address, unsigned char *p, int nbytes );

//HWX_API    bool                 HwxGetSaverDir(            wchar_t *p_saver_dir, int saver_dir_max );

//HWX_API    bool                 HwxIsAbort(                int file_index );
HWX_API    bool                 HwxFileLog(                int file_index, const wchar_t *plogline );
//HWX_API    bool                 HwxFileParseStage(         int file_index, const wchar_t *pstage_text );
//HWX_API    bool                 HwxFileGoAddress(          int file_index, __int64 address );

HWX_API    bool                 HwxFileAddCodeSegment(     int file_index, __int64 address, __int64 foffset, __int64 fsize, __int64 vsize );
HWX_API    bool                 HwxFileAddDataSegment(     int file_index, __int64 address, __int64 foffset, __int64 fsize, __int64 vsize );
HWX_API    bool                 HwxFileSegmentName(        int file_index, __int64 address, const wchar_t *psection_name );
//HWX_API    bool                 HwxFileSegmentCollapse(    int file_index, __int64 address, bool bCollapsed );

HWX_API    bool                 HwxFileMakeCollapse(       int file_index, __int64 address, unsigned long size, bool bCollapsed );

//HWX_API    bool                 HwxFileAddBookmark(        int file_index, __int64 address, const wchar_t *pnote, bookmark_color_t bookmark_color );

//HWX_API    bool                 HwxFileAddName(            int file_index, __int64 address, const wchar_t *pname, const wchar_t literal_group = L' ' );

HWX_API    bool                 HwxFileMakeDec(            int file_index );
HWX_API    bool                 HwxFileMakeHex(            int file_index );
//HWX_API    bool                 HwxFileMakeSign(           int file_index );
//HWX_API    bool                 HwxFileMakeUnsign(         int file_index );
//HWX_API    bool                 HwxFileDataLittleEndian(   int file_index );
//HWX_API    bool                 HwxFileDataBigEndian(      int file_index );
HWX_API    bool                 HwxFileMakePerline(        int file_index, int nPerline );
//HWX_API    bool                 HwxFileMakeFloatFormat(    int file_index, wchar_t *pformat );

HWX_API    bool                 HwxFileSetRunAddress(      int file_index, __int64 address );
//HWX_API    __int64              HwxFileGetRunAddress(      int file_index );

HWX_API    bool                 HwxFileCommentline(        int file_index, __int64 address, const wchar_t *pcomment );
//HWX_API    bool                 HwxFileCommentOnline(      int file_index, __int64 address, const wchar_t *pcomment );
HWX_API    bool                 HwxFileRunCommentline(     int file_index, const wchar_t *pcomment );
HWX_API    bool                 HwxFileRunCommentOnline(   int file_index, const wchar_t *pcomment );

HWX_API    bool                 HwxFileMakeCode(           int file_index, __int64 address );
HWX_API    bool                 HwxFileMakeCodes(          int file_index, __int64 address, unsigned long size );
//HWX_API    bool                 HwxFileRunCode(            int file_index );
//HWX_API    bool                 HwxFileRunCodes(           int file_index, unsigned long size );

HWX_API    bool                 HwxFileMakeDump(           int file_index, __int64 address, unsigned long size, bool bCollapsed );
HWX_API    bool                 HwxFileRunDump(            int file_index, unsigned long size, bool bCollapsed );

//HWX_API    bool                 HwxFileMakeByteline(       int file_index, __int64 address, unsigned long one_llength );
//HWX_API    bool                 HwxFileMakeBytelines(      int file_index, __int64 address, unsigned long one_llength, int nlines, bool bCollapsed );
//HWX_API    bool                 HwxFileRunByteline(        int file_index, unsigned long one_llength );
//HWX_API    bool                 HwxFileRunBytelines(       int file_index, unsigned long one_llength, int nlines, bool bCollapsed );

//HWX_API    bool                 HwxFileMakeByte(           int file_index, __int64 address );
//HWX_API    bool                 HwxFileMakeBytes(          int file_index, __int64 address, int nBytes );
//HWX_API    bool                 HwxFileRunByte(            int file_index );
//HWX_API    bool                 HwxFileRunBytes(           int file_index, int nBytes );
//HWX_API    bool                 HwxFileMakeWord(           int file_index, __int64 address );
HWX_API    bool                 HwxFileMakeWords(          int file_index, __int64 address, int nWords );
HWX_API    bool                 HwxFileRunWord(            int file_index );
HWX_API    bool                 HwxFileRunWords(           int file_index, int nWords );
//HWX_API    bool                 HwxFileMakeDword(          int file_index, __int64 address );
//HWX_API    bool                 HwxFileMakeDwords(         int file_index, __int64 address, int nDwords );
//HWX_API    bool                 HwxFileRunDword(           int file_index );
//HWX_API    bool                 HwxFileRunDwords(          int file_index, int nDwords );
//HWX_API    bool                 HwxFileMakeQword(          int file_index, __int64 address );
//HWX_API    bool                 HwxFileMakeQwords(         int file_index, __int64 address, int nQwords );
//HWX_API    bool                 HwxFileRunQword(           int file_index );
//HWX_API    bool                 HwxFileRunQwords(          int file_index, int nQwords );

//HWX_API    bool                 HwxFileMakeFloat(          int file_index, __int64 address );
//HWX_API    bool                 HwxFileMakeFloats(         int file_index, __int64 address, int nQwords );
//HWX_API    bool                 HwxFileRunFloat(           int file_index );
//HWX_API    bool                 HwxFileRunFloats(          int file_index, int nQwords );
//HWX_API    bool                 HwxFileMakeDouble(         int file_index, __int64 address );
//HWX_API    bool                 HwxFileMakeDoubles(        int file_index, __int64 address, int nQwords );
//HWX_API    bool                 HwxFileRunDouble(          int file_index );
//HWX_API    bool                 HwxFileRunDoubles(         int file_index, int nQwords );

//HWX_API    bool                 HwxFileStringAsciizCP(     int file_index, string_ascii_cp_t ascii_cp );
//HWX_API    bool                 HwxFileMakeStringAsciiz(   int file_index, __int64 address, unsigned long size );
//HWX_API    bool                 HwxFileMakeStringUnicode(  int file_index, __int64 address, unsigned long size );
//HWX_API    bool                 HwxFileRunStringAsciiz(    int file_index, unsigned long nbytes );
//HWX_API    bool                 HwxFileRunStringUnicode(   int file_index, unsigned long nbytes );

//HWX_API    int                  HwxFileAddTable(           int file_index, const wchar_t *ptitle, int ncolumns );
//HWX_API    bool                 HwxFileAddTableHeaders(    int file_index, int table_index, const wchar_t *phdr1, ... );
//HWX_API    bool                 HwxFileAddTableHeadersV(   int file_index, int table_index, const wchar_t **phdrlist );
//HWX_API    bool                 HwxFileAddTableRow(        int file_index, int table_index, const wchar_t *pcol1, ... );
//HWX_API    bool                 HwxFileAddTableRowV(       int file_index, int table_index, const wchar_t **pcollist );
//HWX_API    int                  HwxFileAddTableDetails(    int file_index, int table_index, int row, int col );
//HWX_API    bool                 HwxFileAddTableDetailsRow( int file_index, int table_index, int details_index, const wchar_t *pcol1, const wchar_t *pcol2 );

////////////////////////////////////////////////////////////
